#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#
#pragma ident   "@(#)cluster_status.pl 1.18     01/07/27 SMI"
#
# Cluster status page
#

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Rg;
use Cluster::Cgi;
use Cluster::Node;
use Cluster::Common;
use Cluster::Device;
use Cluster::Transport;
use Sun::Solaris::Utils qw(gettext);

# Create the cluster objects
my $q = new Cluster::Cgi;
my $rg = new Cluster::Rg;
my $node = new Cluster::Node;
my $device = new Cluster::Device;
my $cluster = new Cluster::Common;

# Get cluster info
my $clustername = $cluster->get_clustername();
my $clustersize = $cluster->get_clustersize();

my $title = sprintf(gettext("%s Cluster"), $clustername);

# Print the query header and make the page expire now
print $q->header(-expires => "now");

# Start the document
print $q->start_html({ TITLE  => $title,
		       STYLE  => { 'src' => '/css/clustmgr-style.css'},
		       CLASS  => "default-body",
		       SCRIPT => "top.helpMarker=\"cluster_status-top\"\n" . 
			         $q->jump_menu_jscript() .
				 $q->yoke_jscript("c") 
		       });


##############################################################################
#
# Action Bar Table
#
##############################################################################

# Start the action bar table
$q->start_action_bar_table($title);

# Only provide the links to the topological views if we don't have too many
# nodes in the cluster.
my @viewlinks;

if ($clustersize <= 4 && ($rg->get_num_rgs() <= 4)) {
	@viewlinks = (gettext("Overall Status Tables"),
		      "/cgi-bin/cluster/cluster_status.pl",
		      gettext("Resource Group Topology"),
		      "/cgi-bin/rg/rg_view.pl\?view=topological",
		      gettext("Device Group Topology"),
		      "/cgi-bin/device/device_view.pl\?view=topological",
		      gettext("Transport Topology"),
		      "/cgi-bin/transport/transport_view.pl\?view=topological",
		      );

	$q->view_menu(\@viewlinks, "/cgi-bin/cluster/cluster_status.pl");

} elsif ($clustersize <= 8) {

	@viewlinks = (gettext("Overall Status Tables"),
		      "/cgi-bin/cluster/cluster_status.pl",
		      gettext("Device Group Topology"),
		      "/cgi-bin/device/device_view.pl\?view=topological",
		      gettext("Transport Topology"),
		      "/cgi-bin/transport/transport_view.pl\?view=topological",
		      );

	$q->view_menu(\@viewlinks, "/cgi-bin/cluster/cluster_status.pl");
}

# Print the middle of the action table
$q->mid_action_bar_table();

# Create the actions menu links
my @actionslinks = (gettext("Oracle Database"), "/cgi-bin/ds/oracle.pl",
		    gettext("Apache Web Server"), "/cgi-bin/ds/apache.pl",
		    gettext("iPlanet Web Server"), "/cgi-bin/ds/iplanet.pl",
		    gettext("iPlanet LDAP Server"), "/cgi-bin/ds/ldap.pl",
		    gettext("DNS"), "/cgi-bin/ds/dns.pl",
		    gettext("NFS"), "/cgi-bin/ds/nfs.pl"
		    );

# Create the help links
my @action_helpMarkers = ("cluster_status-ds-oracle",
			  "cluster_status-ds-apache",
			  "cluster_status-ds-iplanet",
			  "cluster_status-ds-ldap",
			  "cluster_status-ds-dns",
			  "cluster_status-ds-nfs");

# Add the help links
$q->load_action_helpMarkers(\@action_helpMarkers);

# Print the actions menu
$q->actions_menu(gettext("Install Data Service"), \@actionslinks);

# End the action bar table
$q->end_action_bar_table();

##############################################################################
#
# Status tables for nodes, resource groups, and global devices. For
# cluster-framework items such as transports and quorum, the users
# will have to drill-down into the menus.
#
##############################################################################

# Print the node status table
$node->node_status_table($q);

# Print the resource groups table
$rg->resource_groups_table($q);

# Print the global devices table
$device->get_commandoutput();
$device->global_devices_table($q);

# End the html
print $q->end_html();

