#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)dns.pl	1.14	01/08/23 SMI"
#

use lib '/opt/SUNWscvw/lib/perl';
use Socket;

use vars qw($alarm);

use Cluster::common;
use Cluster::Common;
use strict;
use Sun::Solaris::Utils qw(gettext);

&do_page("Configure DNS"); # gettext("Configure DNS")
exit;

sub cmd_mainpage {
    my ($i, $gdir);
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));

    &getscrgconfig();
    if (! &checktype("SUNW.dns")) {
	return;
    }
    if (! $errpos) {
	print gettext("To run DNS as a cluster service, the in.named configuration files must be in a global directory.  The installation can create a demo forwarding DNS service, or if you already have in.named configuration files in a global directory, you can use the existing files.");
	print "<br>\n";
	print gettext("Note: any existing in.named servers will be shut down during the configuration process.\n");

	if (! -f "/usr/sbin/in.named") {
	    &errmsg (gettext("/usr/sbin/in.named does not exist"));
	    return;
	}

    }

    if (!&getlh()) {
	return;
    }

    &action_table_start();

    if ($errpos == 101) {
	&errmsg();
    }
    &action_table_row_start(gettext("Network Resource:"));
    &hostnameoption();
    &action_table_row_end();


    if ($errpos == 102) {
	&errmsg();
    }
    &action_table_row_start(gettext("Resource Name:"));
    if ($FORM{"rs"} eq "") {
	# Find a resource name that is not in use
	for ($i = 1; $i<1000; $i++) {
	    $FORM{"rs"} = "dns-$i";
	    if (! defined $rsdesc{$FORM{"rs"}}) {
		last;
	    }
	}
	$FORM{"rs"} = "dns-$i";
    }
    &input("rs");
    &action_table_row_end();


    &action_table_row_start(gettext("Configuration Type:"));
    print "<select name=\"type\">\n";
	&option("type", "demo", gettext("Create demo"));
	&option("type", "exist", gettext("Use existing"));
    print "</select>\n";
    &action_table_supplement(gettext("To run DNS as a cluster service, the in.named configuration files must be in a global directory.  The installation can create a demo forwarding DNS service, or if you already have in.named configuration files in a global directory, you can use the existing files."));
    &action_table_row_end();

    if ($errpos == 5) {
	    &errmsg();
    }

    &action_table_row_start(gettext("Update /etc/resolv.conf, /etc/inet/hosts,<br> and /etc/nsswitch.conf on each host:\n"));
    &checkbox("updatefiles", "active", "", "checked");
    print "Update";
    &action_table_row_end();
    &action_table_end();

    &submit_table_start();
    &submit("cmd_setup", gettext("Continue"));
    &submit_table_end();
    &wizard_end();
}



# Next part of DNS setup
sub cmd_setup {
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    print "<h2>".gettext("DNS setup...")."</h2>\n";

    if ($errpos == 6) {
	    &errmsg();
    }

    &validatelh();

    if ($FORM{"type"} eq "demo") {
	if (! $errpos) {
	    print gettext("You have selected a demo DNS server.  A simple DNS server will be set up on the cluster that will forward requests to an existing nameserver.\n");
	    print gettext("For testing, the demo nameserver will also map name a.b.c to ip 1.2.3.4\n");
	}

	&action_table_start();

	if ($errpos == 1) {
	    &errmsg();
	}
	&action_table_row_start(gettext("Global Directory for Configuration Files:"));
	if ($FORM{"config"} eq "") {
	    $FORM{"config"} = &getglobal."/dns";
	}
	&input("config", 45);
	&action_table_row_end();

	&getnodes();
	my ($lh, $lhrg, $lhrs, $lhmode) = split(/:/, $FORM{"lh"});
	$lh =~ s/ /,/g;
	@lh = split(/,/, $lh);

	if ($errpos == 2) {
	    &errmsg();
	}
	if ($FORM{"ns"} eq "") {
	    $FORM{"ns"} = &finddns();
	}
	&action_table_row_start(gettext("Name of Existing Nameserver:"));
	&input("ns");
	&action_table_row_end();
	&action_table_end();
    } else {
	# Using an existing configuration
	&action_table_start();
	if ($errpos == 1) {
	    &errmsg();
	}
	&action_table_row_start(gettext("Global Path to Configuration Files:"));
	&input("config", 45);
	&action_table_supplement(gettext("(existing in.named config file: named.conf or named.boot)"));
	&action_table_row_end();
	&action_table_end();
    }

    &hidden("lh");
    &hidden("updatefiles");
    &hidden("rs");
    &hidden("type");

    &submit_table_start();
    $FORM{"cmd_setup_1"} = 1;
    &hidden("cmd_setup_1");
    &submit("cmd_setup_1", gettext("Continue"));
    &submit_table_end();
    &wizard_end();
}

# Continue setup
sub cmd_setup_1 {

    my $conf = $FORM{"config"};
    if ($conf eq "") {
	&errpage(gettext("No config file specified"), 1, "cmd_setup");
	return;
    }
    if (&isglobal($conf) != 1) {
	&errpage(gettext("The config file must be in a global directory"), 1, "cmd_setup");
    }

    if ($FORM{"type"} eq "demo") {
    } else {
	if (! -e $conf) {
	    &errpage(gettext("The specified config file does not exist"), 1, "cmd_setup");
	    return;
	}

	if ($conf !~ /named.conf$/ &&
		$conf !~ /named.boot$/) {
	    &errpage(gettext("The configuration must be a named.conf or named.boot file"), 1, "cmd_setup");
	    return;
	}

    }

    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    print "<h2>".gettext("Set up and start HA-DNS")."</h2>\n";

    my ($lh, $lhrg, $lhrs, $lhmode) = split(/:/, $FORM{"lh"});

    if ($FORM{"type"} eq "demo") {
	if ($conf !~ /dns$/) {
	    $conf .= "/dns";
	}
	$conf = &dns_demo($FORM{"ns"}, $conf, $lh.".".&domain());
	if ($conf eq "") {
	    return;
	}
	# Now we should have named set up in the new location
	&okmsg(sprintf(gettext("DNS demo file %s created"), $conf));
	print "<p>\n";
    }

    &getnodes();
    $lh =~ s/ /,/g;
    @lh = split(/,/, $lh);

    my ($dir, $CMD, $err, $i);
    # Modify the config files
    $CMD = "/cgi-bin/ds/dns-script.pl";
    my (%args) = (host => $lh, nodes => join(",", @nodes));
    if ($FORM{"updatefiles"} ne "") {
	my $cluster = new Cluster::Common;
	$err = $cluster->run_parallel($CMD, 1, \%args);
	if ($err) {
	    &errmsg(gettext("File update failed"));
	    return;
	}
    }

#    # Make sure the specified named works.
#    my $cnt = &killit(".usr.sbin.in.named");
#    if ($cnt > 0) {
#	    print "Note: Killed $cnt in.named process(es)\n";
#    }
#
#    print "<p>Testing <tt>/usr/sbin/in.named -c $conf</tt>\n";
#    system("/usr/sbin/in.named -c $conf");
#    my $status = &testnslookup("localhost");
#    if ($status) {
#	&errmsg("The nameserver configuration didn't work\n");
#	print "Please verify that $conf is correct\n";
#	return;
#    }
#    $cnt = &killit(".usr.sbin.in.named");
#    if ($cnt != 0) {
#	    &errmsg("Problem killing in.named process\n");
#	    return;
#    }

    my $mode;
    if ($conf =~ /named.boot/) {
	$mode = "boot";
    } elsif ($conf =~ /named.conf/) {
	$mode = "conf";
    } else {
	&errmsg(sprintf(gettext("Bad conf file name %s"), $conf));
	return;
    }

    # Get directory from config file
    $conf =~ s#/[^/]*$##;

    my ($args);

    $args = "-t SUNW.dns ".
	"-x DNS_mode=\"".$mode."\" -x Confdir_list=\"".$conf."\"";

    my $status = &createservice($args);

    if ($status) {
	return;
    }
    &reloadmenu();
    print $q->_script($q->yoke_jscript("rg-rs-".$FORM{"rs"}));

    print "<p>".sprintf(gettext("You should be able to look up a name with <tt>nslookup</tt> <i>name</i> <tt>%s</tt>\n"), $lh);
    if ($FORM{"type"} eq "demo") {
	print "<p>".sprintf(gettext("For testing, the demo name server should also respond with <tt>1.2.3.4</tt> to <tt>nslookup a.b.c %s</tt>\n"), $lh);
    }
}

# Find a dns server
# Assumes @nodes and @lh are set to "forbidden" servers
sub finddns {
    open(IN, "</etc/resolv.conf") || return "";
    my ($ip, $rawip);
    LOOP: while (<IN>) {
	if (/nameserver\s+(\d+\.\d+\.\d+\.\d+)/) {
	    $ip = $1;
	    my ($name, $aliases, $addrtype, $length, @addrs);
	    $name = iptohost($ip);
	    # Make sure we don't suggest ourself as DNS server
	    my ($i);
	    for ($i = 0; $i <= $#lh; $i++) {
		if ($lh[$i] eq $name) {
		    next LOOP;
		}
	    }
	    for ($i = 0; $i <= $#nodes; $i++) {
		if ($nodes[$i] eq $name) {
		    next LOOP;
		}
	    }
	    if (&testnslookup($name)) {
		# Couldn't access it
		next LOOP;
	    }
	    close(IN);
	    return $name;
	}
    }
    close(IN);
    return "";
}


# Do DNS

# Setup demo, or return "", indicating we've done the error page
sub dns_demo {

    my ($dnsname, $dir, $host) = @_;

    my ($res);
    &killit("in.named");

    $res = &testnslookup($dnsname);
    if ($res) {
	&errpage(gettext("Specified nameserver could not be accessed"),
	    2, "cmd_setup");
    }

    my (@ips) = Cluster::Common::get_ip_addresses(undef, $dnsname);
    if ($#ips < 0) {
	&errpage(sprintf(gettext("Couldn't look up hostname %s"), $dnsname),
	    2, "cmd_setup");
    }

    return &gen_dns_conf($dir, $dnsname, $ips[0], $host);
}

# Make $dir and $dir/named
# Bailout to mainpage on error
sub makedirs {
    my ($dir) = @_;
    my ($s);
    system("/bin/pfexec /usr/bin/mkdir -p -m 0755 $dir/named");
    if (! -d "$dir/named") {
	&errpage(sprintf(gettext("Couldn't create directory %s"), "$dir/named"), 1, "cmd_setup");
    }
}

# Generate a simple dns configuration file
# Dir is the top-level dir to hold named.conf
sub gen_dns_conf {
    my ($file);
    my ($dir, $ns_h, $ns_ip, $host) = @_;

    print "<p>";
    print $q->sprintfn(gettext("Creating named.conf in %1 forwarding to " .
	"nameserver %2 (%3)\n"), $dir, $ns_h, $ns_ip);

    &makedirs($dir);

    $file = "$dir/named/db.b.c";
    if (! open(F, "| /bin/pfexec /bin/tee $file > /dev/null")) {
	&errpage(sprintf(gettext("Couldn't open file %s"), $file), 1, "cmd_setup");
	return "";
    }
    print F << "EOF";
b.c. IN SOA $host. root.$host. (1 10800 3600 604800 86400)
b.c. IN NS $host.

a.b.c. IN A 1.2.3.4
EOF
    if (!close (F)) {
	&errpage(sprintf(gettext("Couldn't open file %s"), $file), 1, "cmd_setup");
    }

    $file = "$dir/named.conf";
    if (! open(F, "| /bin/pfexec /bin/tee $file > /dev/null")) {
	&errpage(sprintf(gettext("Couldn't open file %s"), $file), 1, "cmd_setup");
	return "";
    }
    print F << "EOF";
options {
	directory "$dir/named";
	forwarders { $ns_ip; };
	forward only;
};

zone "b.c" in {
	type master;
	file "db.b.c";
};
EOF

    if (!close (F)) {
	&errpage(sprintf(gettext("Couldn't open file %s"), $file), 1, "cmd_setup");
    }

    return "$dir/named.conf";

}

# Do a pkill on the specified string
sub killit {
	my ($string) = @_;
	if (system("/bin/pfexec /usr/bin/pkill -KILL -U root $string") == 0) {
	    print "<br>";
	    print sprintf(gettext("Stopped %s process.\n"), $string);
	}
}

# See if we can connect to the dns port of the specified server
# Return 0 for ok, 1 for failure
sub testnslookup {
    my ($reply, $found);
    my ($server) = @_;
    socket(DNS, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
    my $addr;
    $addr = sockaddr_in(scalar(getservbyname('domain', 'tcp')),
	inet_aton($server));
    if (connect(DNS, $addr)) {
	close(DNS);
	return 0;
    } else {
	close(DNS);
	return 1;
    }
}
