#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)index.pl	1.26	01/07/27 SMI"
#
# Create the frameset for the three frames. For the menu frame, pass
# along the menu selections.

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Help;
use Cluster::Common;
use Sun::Solaris::Utils qw(gettext);

use constant TRUE  => 0;
use constant FALSE => 1;

# Create the needed objects
my $q = new Cluster::Cgi;
my $cluster = new Cluster::Common;
my $help = new Cluster::Help;

# Declare local vars
my ($TITLE, $BANNER, $MENU, $CONTENT, $clustermode,
    $installed_cluster, $simulated_install, $logout);

# Get cluster state
$clustermode = $cluster->is_clustermode();
$installed_cluster = $cluster->is_installed_cluster();

# Find out if we're simulating installation
if (defined $q->param("mode") && $q->param("mode") eq "install") {
	$simulated_install = TRUE;
} else {
	$simulated_install = FALSE;
}

# Find out if we're logging out
if (defined $q->param("mode") && $q->param("mode") eq "logout") {
	$logout = TRUE;
} else {
	$logout = FALSE;
}

# Determine what frameset to print
if ($installed_cluster == FALSE || $simulated_install == TRUE) {

	# Installation frame targets
	$TITLE = gettext("Sun Cluster Installation");
	$BANNER = "/cgi-bin/banner/banner.pl?mode=install";
	$CONTENT = "/cgi-bin/installation/step0.pl";

} elsif ($installed_cluster == TRUE &&
	 $clustermode == FALSE) {
	
	# Non-clustermode frame targets
	$TITLE = gettext("SunPlex Manager");
	$BANNER = "/cgi-bin/banner/banner.pl";
	$CONTENT = "/cgi-bin/session/nonclustermode.pl";

} elsif ($logout == TRUE) {
	
	# Non-clustermode frame targets
	$TITLE = gettext("SunPlex Manager");
	$BANNER = "/cgi-bin/banner/banner.pl?mode=logout";
	$CONTENT = "/cgi-bin/session/logout.pl";

} else {

	# Get the cluster name
        my $clustername = $cluster->get_clustername();
	$TITLE = sprintf(gettext("%s Cluster Management - Sun Cluster"),
			 $clustername);
    
	# Determine the menu type
	my $menu_type = $q->param("menu");
    
	# Print the right menu link     
	if (defined $menu_type && $menu_type eq "basic") {
		$MENU = "/cgi-bin/menu/menu.pl?mode=basic";
	} else {
		$MENU = "/cgi-bin/menu/menu.pl";
	}

	$BANNER = "/cgi-bin/banner/banner.pl";
	$CONTENT = "/cgi-bin/cluster/cluster_status.pl";
}

# Print the html
print $q->header();
print $q->start_head();
print $q->start_title();
print $TITLE;
print $q->end_title();
print $q->start_script();
print "var helpMarker=\"index\"";
print $help->get_helpMarker_mapping();
print $q->end_script();
print $q->end_head();

# Start the outer frameset
print $q->start_frameset({ ROWS         => '70,*',
			   BORDER       => 2,
			   TITLE	=> $TITLE,
			   FRAMESPACING => 2
			   });

# Print the banner frame
print $q->start_frame({ NAME         => 'banner',
			SRC          => $BANNER,
			TITLE        => gettext('Banner'),
			SCROLLING    => 'NO',
			MARGINHEIGHT => 0,
			MARGINWIDTH  => 0
			});

# Print only if we're in clustermode & not simulating install
if ($clustermode == TRUE && 
    $installed_cluster == TRUE &&
    $simulated_install == FALSE &&
    $logout == FALSE) {

	print $q->start_frameset({ NAME         => 'menu_content', 
				   TITLE        => gettext('Navigation menu'),
				   COLS         => '200,*',
				   BORDER       => 2,
				   FRAMESPACING => 2
				   });
    
	print $q->start_frame({ name         => 'menu',
				title        => gettext('Navigation menu'),
				src          => $MENU,
				scrolling    => 'auto',
				marginheight => 2,
				marginwidth  => 6
				});
}

# Always the content frame
print $q->start_frame({ name         => 'content',
			title        => gettext('Content'),
			src          => $CONTENT,
			scrolling    => 'auto',
			marginheight => 0,
			marginwidth  => 0
			});

# Only end if we have a menu bar
if ($clustermode == TRUE &&
    $installed_cluster == TRUE &&
    $simulated_install == FALSE &&
    $logout == FALSE) {
	print $q->end_frameset();
}

# End the outer frameset
print $q->end_frameset();
print $q->end_html();
