#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)Log.pm	1.13	01/07/27 SMI"
#
# Cluster Log class

package Cluster::Log;
use strict;
use POSIX;
use Cluster::Common;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

# Path to pfexec
my $PFEXEC = "/usr/bin/pfexec";

##############################################################################
#
# Class Methods
#
##############################################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

#
# Print the tail of the specified file
#

sub print_log {
	my ($self, $q, $file, $size) = @_;

	if (!defined $size) {
		$size = 100;
	}

	if (!defined $file) {
		$file = "/var/adm/messages";
	}

	my @output = `$PFEXEC /bin/tail -$size $file`;
	for (my $i=0; $i<$#output; $i++) {
		$output[$i] =~ s/</&lt;/g;
		$output[$i] =~ s/>/&gt;/g;
	}
	print @output;
}

#
# Get the set of logfiles on the specified node
#

sub get_logset_remote($$) {
	my ($self, $node) = @_;
	my $location = "/cgi-bin/log/log_remote.pl?func=logset";
	my @logset = $self->get_http($node,
				     $self->get_http_port(),
				     $location);
	for (my $i=0; $i<=$#logset; $i++) {
		chomp $logset[$i];
	}
	return @logset;
}

#
# Print the log table
#

sub log_table($$$) {
	my ($self, $q, $node, $file, $size) = @_;
	my ($location, @logrows);

	# Get the remote processes
	$location = "/cgi-bin/log/log_remote.pl?file=$file&size=$size";
	@logrows = $self->get_http($node,
				   $self->get_http_port(),
				   $location);

	if (!defined($logrows[0])) {
		# Communcation error
		return;
	}

	# Set the headers for the process status table. 
	my @logheaders = (gettext("Time"), "",
			  "&nbsp;", "",
			  gettext("Log Entry"), "");

	# Determine the number of columns from the headers
	my $numcols = ($#logheaders + 1) / 2;

	# Start the property table
	$q->start_prop_table("", \@logheaders, undef, 1);

	# If we don't have any resources, print a message
	if ($#logrows == -1) {

		# Start a table row and cell with the colspan of the table
		$q->start_prop_tr();
		$q->start_prop_td({ COLSPAN => $numcols });

		# Print the message in table-label-text style
		$q->start_table_text('table-label-text');
		print $q->sprintfn(gettext(
		    "The log file %1 on node %2 is empty."),
			      $file, $node);
		$q->end_table_text();

		# End the cell and row
		$q->end_prop_td();
		$q->end_prop_tr();
	}

	# Cycle through the list of nodes and print the status
	for (my $i=0; $i<=$#logrows; $i++) {

		# Start a table row for the node
		$q->start_prop_tr({ VALIGN => "TOP" });

		# Split the columns
		my @cols = split /(\S+)/, $logrows[$i];

		# Get the time
		my @timeparts = splice @cols, 0, 6;

		# Print the time
		print $q->start_td({ NOWRAP => 1 });
		$q->start_table_text("table-tight-text");
		print @timeparts;
		$q->end_table_text();
		print $q->end_td();

		# Print empty column
		print $q->start_td();
		print "&nbsp;";
		print $q->end_td();

		# Print the log entry
		print $q->start_td();				
		$q->start_table_text("table-tight-text");
		print @cols;
		$q->end_table_text();
		print $q->end_td();
		
		# End the row
		$q->end_prop_tr();

		# If we're not on our last row, print the line row divider
		if ($i < $#logrows) {
			$q->line_row($numcols);
		}
	}

	# End the node status table
	$q->end_prop_table();
}
