#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)Process.pm	1.10	01/07/31 SMI"
#
# Cluster process class

package Cluster::Process;
use strict;
use POSIX;
use Cluster::Common;
use Cluster::RunCommand;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

##############################################################################
#
# Class constructor
#
##############################################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

##############################################################################
#
# Class variables
#
##############################################################################

my $command = new Cluster::RunCommand;

##############################################################################
#
# Class methods
#
##############################################################################

#
# Print the result of ps
#

sub print_processes {
    my ($self, $q) = @_;
    my @output = $command->ps("-ef -o \"nlwp user pid ppid etime time comm\"");

    # Get rid of the header line
    shift @output;

    # Add newlines to the end, for proper sending over http
    foreach my $line (@output) {
	    print "$line\n";
    }
}

#
# Print the process table
#

sub process_table($$$) {
	my ($self, $q, $node) = @_;
	my ($location, @psrows);

	# Get the remote processes
	$location = "/cgi-bin/process/process_remote.pl";
	@psrows = $self->get_http($node,
				  $self->get_http_port(),
				  $location);
			    
	if (!defined($psrows[0])) {
		# Communication error
		return;
	}

	# Set the headers for the process status table. 
	my @psheaders = (gettext("User"), "",
			 gettext("PID"), "",
			 gettext("PPID"), "",
			 gettext("Elapsed"), "",
			 gettext("Time"), "",
			 gettext("Command"), "");

	# Determine the number of columns from the headers
	my $numcols = ($#psheaders + 1) / 2;

	# Start the property table
	$q->start_prop_table("", \@psheaders);

	# Cycle through the list of nodes and print the status
	for (my $i=0; $i<=$#psrows; $i++) {

		# Start a table row for the node
		$q->start_prop_tr();
	
		# Split the columns, handling potential leading spaces
		my @cols = split /(\S+)/, $psrows[$i];
		
		# Print the columns, skipping nlwp and spaces
		for (my $j=3; $j<=$#psheaders+2; $j++) {
			
			# Skip the even cols which are just spaces
			next if (($j % 2) == 0);

			print $q->start_td();				
			$q->start_table_text("table-tight-text");
			if (!defined($cols[$j])) {
				if ($j == $#psheaders+2 && $cols[1] eq "0") {
					# command and defunct
					print "&lt;defunct&gt;";
				} else {
					print "&nbsp;";
				}
			} else {
				$cols[$j] =~ s/</&lt;/g;
				$cols[$j] =~ s/>/&gt;/g;
				print $cols[$j];
			}
			$q->end_table_text();
			print $q->end_td();
		}

		# End the row
		$q->end_prop_tr();

		# If we're not on our last row, print the line row divider
		if ($i < $#psrows) {
			$q->line_row($numcols);
		}
	}

	# End the node status table
	$q->end_prop_table();
}
