#
# Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#ident	"@(#)Script.pm	1.15	02/02/14 SMI"
#
# Banner-related methods

package Cluster::Script;
use strict qw(vars subs);
use IO::Socket;
use IPC::Open2;
use Cluster::Common;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

# This package runs a command.  It de-curses the output from the command,
# looks for prompts, and replies to prompts with answers from an optional
# script.

sub new {
    my $class = shift;
    my $self = {};
    return bless $self, $class;
    return $self;
}

# Execute a command, and reply according to a script.

sub run {
    my ($self, $CMD, $answerscript, $cgi, $outfd) = @_;

    $self->{q} = [];
    $self->{a} = [];
    $self->{alarm_timeout} = 20; # seconds between "waiting" messages
    $self->{max_timeouts} = 30; # number of timeouts before we give up
    $self->{verbose} = 0;
    $self->{alarms} = 0;
    $self->{aptr} = 0;
    $self->{cgi} = $cgi;
    $self->{mode} = 0;

    # Make handle unbuffered
    select((select($outfd), $|=1)[$[]);

    sleep 1;

    # Parse the answer file
    if ($answerscript) {
	    open(IN, "</opt/SUNWscvw/lib/$answerscript")
		    || &bailout("/opt/SUNWscvw/lib/$answerscript");
	    while (<IN>) {
		    chop;
		    my ($q, $a) = split(/[\t=]+/, $_, 2);
		    if ($q =~ /^\s*$/) {
			    next;
		    }
		    if (!defined($a)) {
			$a = "";
		    }
		    push(@{$self->{q}}, $q);
		    push(@{$self->{a}}, $a);
	    }
	    close(IN);
    }

    # Run the command
    my ($toexec) = split(/ /, $CMD);
    if (! -x $toexec) {
	print "ERROR: Can't exec $toexec\n";
	return;
    }
    $self->{pid} = open2(\*RDR, \*WTR, "$CMD 2>&1");

    my $rin = '';
    vec($rin, fileno(RDR), 1) = 1;
    my ($rout, $eout);
    while (1) {
	    my $new = "";
	    my ($nready) = select($rout = $rin, undef, $eout = $rin,
		    $self->{alarm_timeout});
	    if ($nready == 0) {
		# Timeout, do the alarm code.
		my ($quit) = $self->alarm_handler($outfd);
		if ($quit) {
		    $self->cleanup(1, $outfd);
		    return;
		}
		next;
	    }
	    my $s = sysread(RDR, $new, 1000);
	    if (! defined $s) {
		print $outfd "ERROR: sysread failed\n";
		$self->cleanup(0, $outfd);
		return;
	    } elsif ($s == 0) {
		# EOF from command
		$self->cleanup(0, $outfd);
		return;
	    }
	    # Make the stuff we read okay to display
	    $new = $self->clean($new);

	    # Page is the input since the last response.
	    # This will be printed after a timeout, error, etc.
	    $self->{page} .= $new;

	    if ($self->{verbose} == 2 && $self->{mode}==3) {
		# Print all output now, instead of saving in page
		my $t = $new;
		#$t =~ s/[\r\n]/<br>/g;
		print $outfd $t;
		$self->{page} = "";
	    }

	    # Buf is the last line of output so far.
	    $self->{buf} .= $new;
	    # Truncate to last line
	    if ($self->{buf} =~ /[\r\n]+(.*)$/) {
		    $self->{buf} = $1;
	    }
	    $_ = $self->{buf};

	    # Send reply from script
	    if ($self->{page} =~ /ERROR/ || $new =~ /ERROR/ ||
		    $new =~ /not found/ || $new =~ /Undefined/) {
		$self->cleanup(102, $outfd);
		return;
	    }
	    my $err;
	    $err = $self->decode($_, 0, $outfd);
	    if ($err) {
		return;
	    }
    }
}

# Figure out what to reply, according to script
# Print $self->{page} if alarm goes off and at a prompt
# Return 1 to quit, 0 if ok
sub decode {
	my ($self, $request, $alarm, $outfd) = @_;
	my ($i);
	my ($retval) = 0;
	my ($nextquestion) = $self->{q}[$self->{aptr}];
	if ($request =~ /$nextquestion/) {
		#The request is the next expected question; send answer
		$retval = $self->send($request, $self->{a}[$self->{aptr}],
		    1, $alarm, $outfd);
		$self->{aptr}++;
	} elsif ($alarm || $request =~ /\]:/) { # End of line prompt
		# We've timed out, or get an unexpected prompt
		for ($i = 0; $i <= $#{$self->{q}}; $i++) {
			# See if it is in the question list
			if (/$self->{q}[$i]/i) {
				# print $outfd $self->{page};
				# $self->{page} = "";
				# print $outfd "(unexpected order) ";
				$self->{aptr} = $i;
				$retval = $self->send($_,
				    $self->{a}[$self->{aptr}], 1, $alarm,
				    $outfd);
				$self->{aptr}++;
				last;
			}
		}
		if ($alarm && /[\$#%] $/i) {
			print $outfd $self->{page};
			$self->cleanup(0, $outfd);
			return 1;
		}
		if ($i > $#{$self->{q}}) {
			# Got an unexpected prompt, so send newline
			# for lack of anything better to do
			if (! $alarm) {
				#print $outfd "(unexpected) ";
			}
			$retval = $self->send($_, "", 1, $alarm, $outfd);
		}
	}
	return $retval;
}

# Send the appropriate reply
# Print the prompt and reply to $outfd
# Return 1 to quit, 0 otherwise
sub send {
	my ($self, $q, $a, $printit, $alarm, $outfd) = @_;
	my ($n) = "";
	if ($a eq "\$ERROR") {
	    # We've encountered an error
	    $self->cleanup(102, $outfd);
	    return 1;
	} elsif ($a eq "\$SUCCESS") {
	    # We've finished
	    print "Install succeeded\n";
	    $self->cleanup(0, $outfd);
	    return 1;
	}
	if ($a =~ /^\$([A-Z]+)$/) {
		# $VAR substitution; substitute $q{"var"} (lower case);
		$n = $1;
		my ($lcn);
		($lcn = $n) =~ tr/A-Z/a-z/;
		$a = $self->{cgi}->param($lcn);
		if ($a eq "") {
		    print $outfd "**Undefined variable \"$n\"\n";
		}
	}
	# Send the answer
	print WTR $a."\n";
	if ($n =~ "PASSWORD") {
		$a = "******";
	}
	# Print the question and answer
	if ($alarm) {
		print $outfd gettext("waiting\n");
		# Don't clear last page if we had a timeout
	} else {
		if ($printit) {
		    print $outfd "$q <tt>$a</tt>\n";
		}
		$self->{page} = "";
	}
	$self->{buf} = "";
	return 0;
}

# Alarm handler.  This is called it we time out waiting for data.
# Call decode or send \n.  Quit if too many timeouts.
# Return 1 to quit, 0 o
sub alarm_handler {
	my ($self, $outfd) = @_;
	$self->{alarms}++;
	if ($self->{alarms} >= $self->{max_timeouts}) {
	    print $outfd "Timeout\n";
	    print $outfd $self->{page};
	    $self->{page} = "";
	    return 1;
	}
	print $outfd $self->{page};
	$self->{page} = "";
	if ($self->{mode} == 3) {
		my $err;
		$err = $self->decode($_, 1);
		if ($err) {
		    return 1;
		}
	} else {
		$self->send("", "", 1, 1, $outfd);
	}
	return 0;
}

# Remove control characters
sub clean {
	my ($self, $c) = @_;
	$c =~ s/\t/ /g;
	# Remove vt100 sequences
	$c =~ s/\033\[\d*;\d*\w/ /g;	# esc [ n;n X
	$c =~ s/\033\[>\d+\w/ /g;	# esc [ > n X
	$c =~ s/\033\[\d+\w/ /g;	# esc [ n X
	$c =~ s/\033\[\w/ /g;		# esc [ X
	$c =~ s/\033[()][B012]//g;	# esc () B012
	$c =~ tr/\001-\011\013-\037//d;
	return "$c";
}

# Clean up before exiting
# Print $self->{page} if status set
sub cleanup {
	my ($self, $status, $outfd) = @_;
	if ($status) {
		print WTR "\n\003\n\n";
		print $outfd $self->{page};
		$self->{page} = "";
	}
	print WTR "\033quit\n";
	sleep(1);
	kill 9, $self->{pid};
	close(RDR);
	close(WTR);
	if ($status) {
	    print $outfd "\nEXITSTATUS: $status";
	}
	close($outfd);
}

# Module return
1;
